# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os

from oslo_log import log as logging
import oslotest.base
import yaml

from designate.conf.mdns import DEFAULT_MDNS_PORT
from designate import objects
from designate.objects import adapters
from designate.tests import resources

LOG = logging.getLogger(__name__)


class DesignateYAMLAdapterTest(oslotest.base.BaseTestCase):
    def test_yaml_parsing(self):
        file = os.path.join(resources.path, 'pools_yaml/pools.yaml')
        with open(file) as stream:
            xpools = yaml.safe_load(stream)

        for xpool in xpools:
            r_pool = adapters.DesignateAdapter.parse(
                'YAML', xpool, objects.Pool())
            self.assertEqual('default', r_pool.name)
            self.assertEqual('Default PowerDNS 4 Pool', r_pool.description)
            self.assertEqual(2, len(r_pool.ns_records))
            self.assertEqual(1, r_pool.ns_records[0].priority)
            self.assertEqual(2, r_pool.ns_records[1].priority)
            self.assertEqual(
                'ns1-1.example.org.', r_pool.ns_records[0].hostname)
            self.assertEqual(
                'ns1-2.example.org.', r_pool.ns_records[1].hostname)
            self.assertEqual(1, len(r_pool.targets))
            self.assertEqual('pdns4', r_pool.targets[0].type)
            self.assertEqual(
                'PowerDNS 4 Server', r_pool.targets[0].description)
            self.assertEqual(1, len(r_pool.targets[0].masters))
            self.assertEqual('192.0.2.1', r_pool.targets[0].masters[0].host)
            self.assertEqual(DEFAULT_MDNS_PORT,
                             r_pool.targets[0].masters[0].port)
            self.assertEqual(2, len(r_pool.targets[0].options))

            options = {}
            for option in r_pool.targets[0].options:
                options[option.key] = option.value

            self.assertEqual(options['api_endpoint'], 'http://192.0.2.1:8081')
            self.assertEqual(options['api_token'], 'api_key')

            self.assertEqual(1, len(r_pool.also_notifies))
            self.assertEqual('192.0.2.4', r_pool.also_notifies[0].host)
            self.assertEqual(53, r_pool.also_notifies[0].port)
            self.assertEqual(1, len(r_pool.attributes))
            self.assertEqual('type', r_pool.attributes[0].key)
            self.assertEqual('internal', r_pool.attributes[0].value)

    def test_yaml_rendering(self):
        pool_dict = {
            'also_notifies': [
                {
                    'host': '192.0.2.4',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'port': 53,
                }
            ],
            'attributes': [],
            'description': 'Default PowerDNS 4 Pool',
            'id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
            'name': 'default',
            'nameservers': [
                {
                    'host': '192.0.2.2',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'port': 53,
                },
                {
                    'host': '192.0.2.3',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'port': 53,
                }
            ],
            'ns_records': [
                {
                    'hostname': 'ns1-1.example.org.',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'priority': 1,
                },
                {
                    'hostname': 'ns1-2.example.org.',
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'priority': 2,
                }
            ],
            'provisioner': 'UNMANAGED',
            'targets': [
                {
                    'description': 'PowerDNS 4 Server',
                    'masters': [
                        {
                            'host': '192.0.2.1',
                            'pool_target_id': 'd567d569-2d69-41d5-828d-f7054bb10b5c',  # noqa
                            'port': DEFAULT_MDNS_PORT,
                        }
                    ],
                    'options': [
                        {
                            'key': 'api_endpoint',
                            'pool_target_id': 'd567d569-2d69-41d5-828d-f7054bb10b5c',  # noqa
                            'value': 'http://192.0.2.1:8081',  # noqa
                        },
                        {
                            'key': 'api_token',
                            'pool_target_id': 'd567d569-2d69-41d5-828d-f7054bb10b5c',  # noqa
                            'value': 'api_key',  # noqa
                        },
                    ],
                    'pool_id': 'cf2e8eab-76cd-4162-bf76-8aeee3556de0',
                    'type': 'pdns4',
                }
            ],
            'catalog_zone': {
                'catalog_zone_fqdn': 'example.com.',
                'catalog_zone_refresh': 60,
            }
        }

        file = os.path.join(resources.path, 'pools_yaml/sample_output.yaml')
        with open(file) as stream:
            self.assertEqual(
                stream.read(),
                yaml.safe_dump(
                    adapters.DesignateAdapter.render(
                        'YAML', objects.PoolList.from_list([pool_dict])
                    ),
                    default_flow_style=False
                )
            )



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
from unittest import mock

from oslo_log import log as logging
from oslo_utils import timeutils

from designate.central.rpcapi import CentralAPI
from designate.producer import tasks
from designate.storage import sql
from designate.storage.sqlalchemy import tables
from designate.tests import base_fixtures
import designate.tests.functional
from designate.worker import rpcapi as worker_api


LOG = logging.getLogger(__name__)


class DeletedZonePurgeTest(designate.tests.functional.TestCase):
    number_of_zones = 20
    batch_size = 5
    time_threshold = 24 * 60 * 60

    def setUp(self):
        super().setUp()
        self.config(
            time_threshold=self.time_threshold,
            batch_size=self.batch_size,
            group='producer_task:zone_purge'
        )
        self.purge_task_fixture = self.useFixture(
            base_fixtures.ZoneManagerTaskFixture(tasks.DeletedZonePurgeTask)
        )

    def _create_deleted_zone(self, name, mock_deletion_time):
        # Create a zone and set it as deleted
        zone = self.create_zone(name=name)
        self._delete_zone(zone, mock_deletion_time)

    def _fetch_all_zones(self):
        # Fetch all zones including deleted ones.
        query = tables.zones.select()
        with sql.get_read_session() as session:
            return session.execute(query).fetchall()

    def _delete_zone(self, zone, mock_deletion_time):
        # Set a zone as deleted
        zid = zone.id.replace('-', '')
        query = tables.zones.update().where(tables.zones.c.id == zid).values(
                action='NONE',
                deleted=zid,
                deleted_at=mock_deletion_time,
                status='DELETED',
        )

        with sql.get_write_session() as session:
            pxy = session.execute(query)
            self.assertEqual(1, pxy.rowcount)

    def _create_deleted_zones(self):
        # Create a number of deleted zones in the past days.
        now = timeutils.utcnow()
        for index in range(self.number_of_zones):
            if index % 2 == 0:
                age = self.time_threshold * 2
            else:
                age = self.time_threshold // 2
            delta = datetime.timedelta(seconds=age)
            deletion_time = now - delta
            name = 'example%d.org.' % index
            self._create_deleted_zone(name, deletion_time)

    def test_purge_zones(self):
        # Create X zones, run producer, check if half of the zones
        # are remaining.
        self.config(quota_zones=self.number_of_zones)
        self._create_deleted_zones()

        for remaining in range(5):
            self.purge_task_fixture.task()

        remaning_zones = self._fetch_all_zones()
        self.assertEqual(len(remaning_zones), self.number_of_zones // 2)


class PeriodicGenerateDelayedNotifyTaskTest(
        designate.tests.functional.TestCase):
    number_of_zones = 20
    batch_size = 5

    def setUp(self):
        super().setUp()
        self.config(quota_zones=self.number_of_zones)
        self.config(
            batch_size=self.batch_size,
            group='producer_task:delayed_notify'
        )
        self.generate_delayed_notify_task_fixture = self.useFixture(
            base_fixtures.ZoneManagerTaskFixture(
                tasks.PeriodicGenerateDelayedNotifyTask
            )
        )

    def _fetch_all_zones(self):
        # Fetch all zones including deleted ones.
        return self._fetch_zones(tables.zones.select())

    def _fetch_zones(self, query):
        # Fetch zones including deleted ones.
        with sql.get_read_session() as session:
            return session.execute(query).fetchall()

    def _create_zones(self):
        # Create a number of zones; half of them with delayed_notify set.
        for index in range(self.number_of_zones):
            name = 'example%d.org.' % index
            delayed_notify = (index % 2 == 0)
            self.create_zone(
                name=name,
                delayed_notify=delayed_notify,
            )

    def test_generate_delayed_notify_zones(self):
        # Create zones and set some of them as pending update.
        self._create_zones()

        zones = self._fetch_all_zones()
        self.assertEqual(self.number_of_zones, len(zones))

        for remaining in reversed(range(0,
                                        self.number_of_zones // 2,
                                        self.batch_size)):
            self.generate_delayed_notify_task_fixture.task()

            zones = self._fetch_zones(tables.zones.select().where(
                tables.zones.c.delayed_notify))

            self.assertEqual(
                remaining, len(zones),
                message='Remaining zones: %s' % zones
            )


class PeriodicIncrementSerialTaskTest(designate.tests.functional.TestCase):
    number_of_zones = 20
    batch_size = 20

    def setUp(self):
        super().setUp()
        self.worker_api = mock.Mock()
        mock.patch.object(worker_api.WorkerAPI, 'get_instance',
                          return_value=self.worker_api).start()
        self.config(quota_zones=self.number_of_zones)
        self.config(
            batch_size=self.batch_size,
            group='producer_task:increment_serial'
        )
        self.increment_serial_task_fixture = self.useFixture(
            base_fixtures.ZoneManagerTaskFixture(
                tasks.PeriodicIncrementSerialTask
            )
        )

    def _create_zones(self):
        for index in range(self.number_of_zones):
            name = 'example%d.org.' % index
            increment_serial = (index % 2 == 0)
            delayed_notify = (index % 4 == 0)
            self.create_zone(
                name=name,
                increment_serial=increment_serial,
                delayed_notify=delayed_notify,
            )

    def test_increment_serial(self):
        self._create_zones()

        self.increment_serial_task_fixture.task()

        self.worker_api.update_zone.assert_called()
        self.assertEqual(5, self.worker_api.update_zone.call_count)


class PeriodicSecondaryRefreshTaskTest(designate.tests.functional.TestCase):

    def setUp(self):
        super().setUp()
        task_class = tasks.PeriodicSecondaryRefreshTask
        fixture = base_fixtures.ZoneManagerTaskFixture(task_class)
        self.periodic_secondary_refresh_task_fixture = self.useFixture(fixture)

    def _create_zones(self):
        refresh = 3600
        need_refresh = datetime.timedelta(seconds=refresh)
        do_not_need_refresh = need_refresh / 2
        now = timeutils.utcnow(True)
        all_transferred_at = (
            None,
            now - need_refresh,
            now - do_not_need_refresh
        )
        email = 'service_central_managed_resource_email@example.org'
        for name_index, transferred_at in enumerate(all_transferred_at):
            self.create_zone(
                name='example_%d.org.' % name_index,
                email=email,
                type='SECONDARY',
                refresh=refresh,
                transferred_at=transferred_at
            )

    @mock.patch.object(CentralAPI, 'xfr_zone')
    def test_refresh_secondary_zone(self, mock_xfr_zone):
        self._create_zones()
        self.periodic_secondary_refresh_task_fixture.task()
        mock_xfr_zone.assert_called_once()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslotest.base

from designate.manage import base


class TestManageBase(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_init(self):
        manage_base = base.Commands()
        self.assertEqual('designate-manage', manage_base.context.request_id)
        self.assertEqual([''], manage_base.output_message)

    @mock.patch('builtins.print')
    def test_print_result(self, mock_print):
        manage_base = base.Commands()
        manage_base.output_message.append('foo')
        manage_base.output_message.append('bar')

        manage_base._print_result()

        self.assertIn(mock.call(''), mock_print.call_args_list)
        self.assertIn(mock.call('foo'), mock_print.call_args_list)
        self.assertIn(mock.call('bar'), mock_print.call_args_list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading
from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
import designate.exceptions
from designate.network_api import base


CONF = designate.conf.CONF

SERVICE_CATALOG = [
    {
        'endpoints': [
            {
                'adminURL': 'admin1',
                'region': 'RegionOne',
                'internalURL': 'internal1',
                'publicURL': 'public1'
            }
        ],
        'type': 'dns',
        'name': 'foo1'
    },
    {
        'endpoints': [
            {
                'adminURL': 'admin2',
                'region': 'RegionTwo',
                'internalURL': 'internal2',
                'publicURL': 'public2'
            }
        ],
        'type': 'dns',
        'name': 'foo2'
    },
    {
        'endpoints': [
            {
                'adminURL': 'admin3',
                'region': 'RegionTwo',
                'internalURL': 'internal3',
                'publicURL': 'public3'
            }
        ],
        'type': 'network',
        'name': 'foo2'
    },
]


class NetworkEndpointsTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

        self.base = base.NetworkAPI()

    def test_endpoints_from_config(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3', 'RegionFour|public4'],
            'network_api:neutron'
        )

        result = self.base._endpoints(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
            config_section='network_api:neutron',
        )

        self.assertEqual(
            [('public3', 'RegionThree'), ('public4', 'RegionFour')], result
        )

    def test_endpoints_from_config_with_region(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3', 'RegionFour|public4'],
            'network_api:neutron'
        )

        result = self.base._endpoints(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
            region='RegionFour',
            config_section='network_api:neutron',
        )

        self.assertEqual(
            [('public4', 'RegionFour')], result
        )

    def test_endpoints_from_catalog(self):
        result = self.base._endpoints(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
            config_section='network_api:neutron',
        )

        self.assertEqual(
            [('public1', 'RegionOne'), ('public2', 'RegionTwo')], result
        )

    def test_endpoints_from_catalog_with_region(self):
        result = self.base._endpoints(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
            region='RegionOne',
            config_section='network_api:neutron',
        )

        self.assertEqual(
            [('public1', 'RegionOne')], result
        )

    def test_no_endpoints_or_service_catalog_available(self):
        self.assertRaisesRegex(
            designate.exceptions.ConfigurationError,
            'No service_catalog and no configured endpoints',
            self.base._endpoints,
            service_catalog=None,
            service_type='dns',
            endpoint_type='publicURL',
            config_section='network_api:neutron',
        )


class NetworkEndpointsFromConfigTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

        self.base = base.NetworkAPI()

    def test_endpoint(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3'], 'network_api:neutron'
        )

        result = self.base.endpoints_from_config(
            CONF['network_api:neutron'].endpoints,
        )

        self.assertEqual(
            [('public3', 'RegionThree')], result
        )

    def test_endpoints(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3', 'RegionFour|public4'],
            'network_api:neutron'
        )

        result = self.base.endpoints_from_config(
            CONF['network_api:neutron'].endpoints,
        )

        self.assertEqual(
            [('public3', 'RegionThree'), ('public4', 'RegionFour')], result
        )

    def test_endpoint_from_region(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3', 'RegionFour|public4'],
            'network_api:neutron'
        )

        result = self.base.endpoints_from_config(
            CONF['network_api:neutron'].endpoints,
            region='RegionFour',
        )

        self.assertEqual(
            [('public4', 'RegionFour')], result
        )

    def test_endpoint_from_region_not_found(self):
        CONF.set_override(
            'endpoints', ['RegionThree|public3', 'RegionThree|public4'],
            'network_api:neutron'
        )

        self.assertRaisesRegex(
            designate.exceptions.ConfigurationError,
            'Endpoints are not correctly configured',
            self.base.endpoints_from_config,
            CONF['network_api:neutron'].endpoints,
            region='RegionFive',
        )

    def test_endpoint_empty_list(self):
        CONF.set_override(
            'endpoints', [],
            'network_api:neutron'
        )

        self.assertRaisesRegex(
            designate.exceptions.ConfigurationError,
            'Endpoints are not correctly configured',
            self.base.endpoints_from_config,
            CONF['network_api:neutron'].endpoints,
        )


class NetworkEndpointsFromCatalogTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

        self.base = base.NetworkAPI()

    def test_endpoints(self):
        result = self.base.endpoints_from_catalog(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
        )

        self.assertEqual(
            [('public1', 'RegionOne'), ('public2', 'RegionTwo')], result
        )

    def test_endpoint_from_region(self):
        result = self.base.endpoints_from_catalog(
            service_catalog=SERVICE_CATALOG,
            service_type='dns',
            endpoint_type='publicURL',
            region='RegionTwo',
        )

        self.assertEqual(
            [('public2', 'RegionTwo')], result
        )

    def test_endpoint_region_not_found(self):
        self.assertRaises(
            designate.exceptions.NetworkEndpointNotFound,
            self.base.endpoints_from_catalog,
            service_type='dns',
            endpoint_type='publicURL',
            region='RegionSix',
            service_catalog=SERVICE_CATALOG,
        )

    def test_no_endpoints_found(self):
        self.assertRaises(
            designate.exceptions.NetworkEndpointNotFound,
            self.base.endpoints_from_catalog,
            service_type='compute',
            endpoint_type='publicURL',
            service_catalog=SERVICE_CATALOG,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import oslotest.base

from designate.common import constants


class TestRegex(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_tldname_is_valid(self):
        valid_tldnames = [
            'com',
            'net',
            'org',
            'co.uk',
        ]

        for tldname in valid_tldnames:
            self.assertTrue(constants.RE_TLDNAME.match(tldname))

    def test_tldname_is_not_valid(self):
        invalid_tldnames = [
            # Invalid Formats
            'com.',
            '.com',
            # Trailing newline - Bug 1471158
            "com\n",
        ]

        for tldname in invalid_tldnames:
            self.assertFalse(constants.RE_TLDNAME.match(tldname))



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from io import StringIO
import os
import sys

from alembic import command as alembic_command
from alembic.config import Config
from oslo_log import log as logging

import designate.conf
from designate.manage import base

CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class DatabaseCommands(base.Commands):
    def _get_alembic_config(self, db_url=None, stringio_buffer=sys.stdout):
        alembic_dir = os.path.join(os.path.dirname(__file__),
                                   os.pardir, 'storage/sqlalchemy')
        alembic_cfg = Config(os.path.join(alembic_dir, 'alembic.ini'),
                             stdout=stringio_buffer)
        alembic_cfg.set_main_option(
            'script_location', 'designate.storage.sqlalchemy:alembic')
        if db_url:
            alembic_cfg.set_main_option('sqlalchemy.url', db_url)
        else:
            alembic_cfg.set_main_option('sqlalchemy.url',
                                        CONF['storage:sqlalchemy'].connection)
        return alembic_cfg

    def current(self, db_url=None, stringio_buffer=sys.stdout):
        alembic_command.current(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=stringio_buffer))

    def heads(self, db_url=None, stringio_buffer=sys.stdout):
        alembic_command.heads(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=stringio_buffer))

    def history(self, db_url=None, stringio_buffer=sys.stdout):
        alembic_command.history(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=stringio_buffer))

    def version(self, db_url=None):
        # Using StringIO buffers here to keep the command output as similar
        # as it was before the migration to alembic.
        current_buffer = StringIO()
        latest_buffer = StringIO()
        alembic_command.current(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=current_buffer))
        current = current_buffer.getvalue().replace('\n', ' ')
        current_buffer.close()
        alembic_command.heads(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=latest_buffer))
        latest = latest_buffer.getvalue().replace('\n', ' ')
        latest_buffer.close()
        print(f'Current: {current} Latest: {latest}')

    def sync(self, db_url=None, stringio_buffer=sys.stdout):
        alembic_command.upgrade(
            self._get_alembic_config(db_url=db_url,
                                     stringio_buffer=stringio_buffer), 'head')

    @base.args('revision', default='head', nargs='?',
               help='The revision identifier to upgrade to.')
    def upgrade(self, revision, db_url=None, stringio_buffer=sys.stdout):
        alembic_command.upgrade(
            self._get_alembic_config(
                db_url=db_url, stringio_buffer=stringio_buffer), revision)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
# Base Adapter Class
from designate.objects.adapters.base import DesignateAdapter  # noqa
# API v2
from designate.objects.adapters.api_v2.blacklist import BlacklistAPIv2Adapter, BlacklistListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone import ZoneAPIv2Adapter, ZoneListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_attribute import ZoneAttributeAPIv2Adapter, ZoneAttributeListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_master import ZoneMasterAPIv2Adapter, ZoneMasterListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.floating_ip import FloatingIPAPIv2Adapter, FloatingIPListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.record import RecordAPIv2Adapter, RecordListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.recordset import RecordSetAPIv2Adapter, RecordSetListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.pool import PoolAPIv2Adapter, PoolListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.pool_attribute import PoolAttributeAPIv2Adapter, PoolAttributeListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.pool_ns_record import PoolNsRecordAPIv2Adapter, PoolNsRecordListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.tld import TldAPIv2Adapter, TldListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.tsigkey import TsigKeyAPIv2Adapter, TsigKeyListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.quota import QuotaAPIv2Adapter, QuotaListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.service_status import ServiceStatusAPIv2Adapter, ServiceStatusListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_transfer_accept import ZoneTransferAcceptAPIv2Adapter, ZoneTransferAcceptListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_transfer_request import ZoneTransferRequestAPIv2Adapter, ZoneTransferRequestListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.validation_error import ValidationErrorAPIv2Adapter, ValidationErrorListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_import import ZoneImportAPIv2Adapter, ZoneImportListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.zone_export import ZoneExportAPIv2Adapter, ZoneExportListAPIv2Adapter  # noqa
from designate.objects.adapters.api_v2.shared_zone import SharedZoneAPIv2Adapter, SharedZoneListAPIv2Adapter  # noqa
# YAML

from designate.objects.adapters.yaml.pool import PoolYAMLAdapter, PoolListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_attribute import PoolAttributeYAMLAdapter, PoolAttributeListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_also_notify import PoolAlsoNotifyYAMLAdapter, PoolAlsoNotifyListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_catalog_zone import PoolCatalogZoneYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_nameserver import PoolNameserverYAMLAdapter, PoolNameserverListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_ns_record import PoolNsRecordYAMLAdapter, PoolNsRecordListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_target import PoolTargetYAMLAdapter, PoolTargetListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_target_master import PoolTargetMasterYAMLAdapter, PoolTargetMasterListYAMLAdapter  # noqa
from designate.objects.adapters.yaml.pool_target_option import PoolTargetOptionYAMLAdapter, PoolTargetOptionListYAMLAdapter  # noqa



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from openstack import exceptions as sdk_exceptions
from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.conf
from designate import context
from designate import exceptions
from designate.network_api import get_network_api
from designate.network_api import neutron
from designate import version


CONF = designate.conf.CONF


class NeutronNetworkAPITest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

        CONF.set_override(
            'endpoints', ['RegionOne|http://192.0.2.5:9696'],
            'network_api:neutron'
        )
        self.ca_certificates_file = 'fake_ca_cert_file'
        self.client_certificate_file = 'fake_client_cert_file'
        CONF.set_override('client_certificate_file',
                          self.client_certificate_file,
                          'network_api:neutron')
        self.neutron_timeout = 100
        CONF.set_override('timeout', self.neutron_timeout,
                          'network_api:neutron')

        self.api = get_network_api('neutron')
        self.context = context.DesignateContext(
            user_id='12345', project_id='54321',
        )

    @mock.patch('keystoneauth1.token_endpoint.Token')
    @mock.patch('keystoneauth1.session.Session')
    @mock.patch('openstack.connection.Connection')
    def test_get_client(self, mock_client, mock_session, mock_token):
        auth_token_mock = mock.MagicMock()
        mock_token.return_value = auth_token_mock

        user_session_mock = mock.MagicMock()
        mock_session.return_value = user_session_mock

        connection_mock = mock.MagicMock()
        mock_client.return_value = connection_mock

        self.context = context.DesignateContext(
            user_id='12345', project_id='54321', auth_token='token',
        )
        endpoint = 'http://192.0.2.5:9696'

        result = neutron.get_client(self.context, endpoint)

        mock_token.assert_called_once_with(endpoint, self.context.auth_token)

        mock_session.assert_called_once_with(
            auth=auth_token_mock, verify=True,
            cert=self.client_certificate_file, timeout=self.neutron_timeout,
            app_name='designate',
            app_version=version.version_info.version_string())

        self.assertEqual(connection_mock, result)

        # Test with CA certs file configuration
        mock_token.reset_mock()
        mock_session.reset_mock()

        CONF.set_override('ca_certificates_file', self.ca_certificates_file,
                          'network_api:neutron')

        result = neutron.get_client(self.context, endpoint)

        mock_token.assert_called_once_with(endpoint, self.context.auth_token)

        mock_session.assert_called_once_with(
            auth=auth_token_mock, verify=self.ca_certificates_file,
            cert=self.client_certificate_file, timeout=self.neutron_timeout,
            app_name='designate',
            app_version=version.version_info.version_string())

        self.assertEqual(connection_mock, result)

        # Test with insecure configuration
        mock_token.reset_mock()
        mock_session.reset_mock()

        CONF.set_override('insecure', True, 'network_api:neutron')

        result = neutron.get_client(self.context, endpoint)

        mock_token.assert_called_once_with(endpoint, self.context.auth_token)

        mock_session.assert_called_once_with(
            auth=auth_token_mock, verify=False,
            cert=self.client_certificate_file, timeout=self.neutron_timeout,
            app_name='designate',
            app_version=version.version_info.version_string())

        self.assertEqual(connection_mock, result)

    @mock.patch('designate.network_api.neutron.get_client')
    def test_list_floatingips(self, get_client):
        driver = mock.Mock()
        driver.network.ips.return_value = [
            {
                'id': '123',
                'floating_ip_address': '192.0.2.100',
                'region': 'RegionOne'
            },
            {
                'id': '456',
                'floating_ip_address': '192.0.2.200',
                'region': 'RegionOne'
            },
        ]
        get_client.return_value = driver

        self.assertEqual(2, len(self.api.list_floatingips(self.context)))

    @mock.patch('designate.network_api.neutron.get_client')
    def test_list_floatingips_unauthorized(self, get_client):
        driver = mock.Mock()
        driver.network.ips.side_effect = sdk_exceptions.HttpException
        get_client.return_value = driver

        self.assertEqual(0, len(self.api.list_floatingips(self.context)))

    @mock.patch('designate.network_api.neutron.get_client')
    def test_list_floatingips_communication_failure(self, get_client):
        driver = mock.Mock()
        driver.network.ips.side_effect = (
            Exception
        )
        get_client.return_value = driver

        self.assertRaises(
            exceptions.NeutronCommunicationFailure,
            self.api.list_floatingips, self.context
        )



# Copyright 2020 Cloudification GmbH. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from urllib import parse

from designate import objects
from designate.objects.adapters.api_v2 import base


class SharedZoneAPIv2Adapter(base.APIv2Adapter):

    ADAPTER_OBJECT = objects.SharedZone

    MODIFICATIONS = {
        'fields': {
            "id": {},
            "zone_id": {},
            "project_id": {},
            "target_project_id": {'immutable': True},
            "created_at": {},
            "updated_at": {},
        },
        'options': {
            'links': True,
            'resource_name': 'shared_zone',
            'collection_name': 'shared_zones',
        }
    }

    @classmethod
    def render_object(cls, object, *args, **kwargs):
        obj = super().render_object(
            object, *args, **kwargs)

        if obj['zone_id'] is not None:
            obj['links']['self'] = (
                '{}/v2/zones/{}/shares/{}'.format(
                    cls._get_base_url(kwargs['request']), obj['zone_id'],
                    obj['id']))
            obj['links']['zone'] = (
                '{}/v2/zones/{}'.format(cls._get_base_url(kwargs['request']),
                                        obj['zone_id']))
        return obj


class SharedZoneListAPIv2Adapter(base.APIv2Adapter):

    ADAPTER_OBJECT = objects.SharedZoneList

    MODIFICATIONS = {
        'options': {
            'links': True,
            'resource_name': 'shared_zone',
            'collection_name': 'shared_zones',
        }
    }

    @classmethod
    def _get_collection_href(cls, request, extra_params=None):
        params = request.GET

        if extra_params is not None:
            params.update(extra_params)

        base_uri = cls._get_base_url(request)

        href = '{}{}?{}'.format(
            base_uri,
            request.path,
            parse.urlencode(params))

        return href.rstrip('?')



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslo_messaging as messaging

from designate.common.decorators import rpc as rpc_decorator
from designate.common import profiler
import designate.conf
from designate import rpc


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)

CENTRAL_API = None


def reset():
    global CENTRAL_API
    CENTRAL_API = None


@profiler.trace_cls("rpc")
@rpc_decorator.rpc_logging(LOG, 'central')
class CentralAPI:
    """
    Client side of the central RPC API.

    API version history:

        1.0 - Initial version
        1.1 - Add new finder methods
        1.2 - Add get_tenant and get_tenants
        1.3 - Add get_absolute_limits
        2.0 - Renamed most get_resources to find_resources
        2.1 - Add quota methods
        3.0 - RecordSet Changes
        3.1 - Add floating ip ptr methods
        3.2 - TLD Api changes
        3.3 - Add methods for blacklisted domains
        4.0 - Create methods now accept designate objects
        4.1 - Add methods for server pools
        4.2 - Add methods for pool manager integration
        4.3 - Added Zone Transfer Methods
        5.0 - Remove dead server code
        5.1 - Add xfr_zone
        5.2 - Add Zone Import methods
        5.3 - Add Zone Export method
        5.4 - Add asynchronous Zone Export methods
        5.5 - Add deleted zone purging task
        5.6 - Changed 'purge_zones' function args
        6.0 - Renamed domains to zones
        6.1 - Add ServiceStatus methods
        6.2 - Changed 'find_recordsets' method args
        6.3 - Changed 'update_status' method args
        6.4 - Removed unused record and diagnostic methods
        6.5 - Removed additional unused methods
        6.6 - Add methods for shared zones
        6.7 - Add increment_zone_serial
        6.8 - Add managed recordset methods
        6.9 - Removed unused methods
        6.10 - Add Zone Pool Move method
    """
    RPC_API_VERSION = '6.10'

    # This allows us to mark some methods as not logged.
    # This can be for a few reasons - some methods my not actually call over
    # RPC, or may be so noisy that logging them is not useful
    # This should be an array of strings that match the function names
    RPC_LOGGING_DISALLOW = ['update_service_status']

    def __init__(self, topic=None):
        self.topic = topic if topic else CONF['service:central'].topic

        target = messaging.Target(topic=self.topic,
                                  version=self.RPC_API_VERSION)
        self.client = rpc.get_client(target, version_cap='6.10')

    @classmethod
    def get_instance(cls):
        """
        The rpc.get_client() which is called upon the API object initialization
        will cause a assertion error if the designate.rpc.TRANSPORT isn't setup
        by rpc.init() before.

        This fixes that by creating the rpcapi when demanded.
        """
        global CENTRAL_API
        if not CENTRAL_API:
            CENTRAL_API = cls()
        return CENTRAL_API

    # Misc Methods
    def get_absolute_limits(self, context):
        return self.client.call(context, 'get_absolute_limits')

    # Quota Methods
    def get_quotas(self, context, tenant_id):
        return self.client.call(context, 'get_quotas', tenant_id=tenant_id)

    def set_quota(self, context, tenant_id, resource, hard_limit):
        return self.client.call(context, 'set_quota', tenant_id=tenant_id,
                                resource=resource, hard_limit=hard_limit)

    def reset_quotas(self, context, tenant_id):
        return self.client.call(context, 'reset_quotas', tenant_id=tenant_id)

    # TSIG Key Methods
    def create_tsigkey(self, context, tsigkey):
        return self.client.call(context, 'create_tsigkey', tsigkey=tsigkey)

    def find_tsigkeys(self, context, criterion=None, marker=None, limit=None,
                      sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_tsigkeys', criterion=criterion,
                                marker=marker, limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def get_tsigkey(self, context, tsigkey_id):
        return self.client.call(context, 'get_tsigkey', tsigkey_id=tsigkey_id)

    def update_tsigkey(self, context, tsigkey):
        return self.client.call(context, 'update_tsigkey', tsigkey=tsigkey)

    def delete_tsigkey(self, context, tsigkey_id):
        return self.client.call(context, 'delete_tsigkey',
                                tsigkey_id=tsigkey_id)

    # Tenant Methods
    def find_tenants(self, context):
        return self.client.call(context, 'find_tenants')

    def get_tenant(self, context, tenant_id):
        return self.client.call(context, 'get_tenant', tenant_id=tenant_id)

    # Zone Methods
    def increment_zone_serial(self, context, zone):
        return self.client.call(context, 'increment_zone_serial', zone=zone)

    def create_zone(self, context, zone):
        return self.client.call(context, 'create_zone', zone=zone)

    def get_zone(self, context, zone_id):
        return self.client.call(context, 'get_zone', zone_id=zone_id)

    def get_zone_ns_records(self, context, zone_id):
        return self.client.call(context, 'get_zone_ns_records',
                                zone_id=zone_id)

    def find_zones(self, context, criterion=None, marker=None, limit=None,
                   sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_zones', criterion=criterion,
                                marker=marker, limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def update_zone(self, context, zone, increment_serial=True):
        return self.client.call(context, 'update_zone', zone=zone,
                                increment_serial=increment_serial)

    def delete_zone(self, context, zone_id):
        return self.client.call(context, 'delete_zone', zone_id=zone_id)

    def purge_zones(self, context, criterion, limit=None):
        return self.client.call(context, 'purge_zones',
                                criterion=criterion, limit=limit)

    def pool_move_zone(self, context, zone_id, target_pool_id):
        return self.client.call(context, 'pool_move_zone',
                                zone_id=zone_id,
                                target_pool_id=target_pool_id)

    # Shared Zone methods
    def share_zone(self, context, zone_id, shared_zone):
        return self.client.call(context, 'share_zone', zone_id=zone_id,
                                shared_zone=shared_zone)

    def unshare_zone(self, context, zone_id, zone_share_id):
        return self.client.call(context, 'unshare_zone',
                                zone_id=zone_id, zone_share_id=zone_share_id)

    def find_shared_zones(self, context, criterion=None, marker=None,
                          limit=None, sort_key=None, sort_dir=None):
        return self.client.call(
            context, 'find_shared_zones', criterion=criterion, marker=marker,
            limit=limit, sort_key=sort_key, sort_dir=sort_dir)

    def get_shared_zone(self, context, zone_id, zone_share_id):
        return self.client.call(
            context, 'get_shared_zone', zone_id=zone_id,
            zone_share_id=zone_share_id
        )

    # TLD Methods
    def create_tld(self, context, tld):
        return self.client.call(context, 'create_tld', tld=tld)

    def find_tlds(self, context, criterion=None, marker=None, limit=None,
                  sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_tlds', criterion=criterion,
                                marker=marker, limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def get_tld(self, context, tld_id):
        return self.client.call(context, 'get_tld', tld_id=tld_id)

    def update_tld(self, context, tld):
        return self.client.call(context, 'update_tld', tld=tld)

    def delete_tld(self, context, tld_id):
        return self.client.call(context, 'delete_tld', tld_id=tld_id)

    # RecordSet Methods
    def create_recordset(self, context, zone_id, recordset):
        return self.client.call(context, 'create_recordset',
                                zone_id=zone_id, recordset=recordset)

    def get_recordset(self, context, zone_id, recordset_id):
        return self.client.call(context, 'get_recordset', zone_id=zone_id,
                                recordset_id=recordset_id)

    def find_recordsets(self, context, criterion=None, marker=None, limit=None,
                        sort_key=None, sort_dir=None, force_index=False):
        return self.client.call(context, 'find_recordsets',
                                criterion=criterion, marker=marker,
                                limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir, force_index=force_index)

    def create_managed_records(self, context, zone_id, records_values,
                               recordset_values):
        return self.client.call(context, 'create_managed_records',
                                zone_id=zone_id, records_values=records_values,
                                recordset_values=recordset_values)

    def delete_managed_records(self, context, zone_id=None, criterion=None):
        return self.client.call(context, 'delete_managed_records',
                                zone_id=zone_id,
                                criterion=criterion)

    def export_zone(self, context, zone_id):
        return self.client.call(context, 'export_zone', zone_id=zone_id)

    def update_recordset(self, context, recordset, increment_serial=True):
        return self.client.call(context, 'update_recordset',
                                recordset=recordset,
                                increment_serial=increment_serial)

    def delete_recordset(self, context, zone_id, recordset_id,
                         increment_serial=True):
        return self.client.call(context, 'delete_recordset',
                                zone_id=zone_id,
                                recordset_id=recordset_id,
                                increment_serial=increment_serial)

    # Misc. Report combining counts for tenants, zones and records
    def count_report(self, context, criterion=None):
        return self.client.call(context, 'count_report', criterion=criterion)

    def list_floatingips(self, context):
        return self.client.call(context, 'list_floatingips')

    def get_floatingip(self, context, region, floatingip_id):
        return self.client.call(context, 'get_floatingip', region=region,
                                floatingip_id=floatingip_id)

    def update_floatingip(self, context, region, floatingip_id, values):
        return self.client.call(context, 'update_floatingip', region=region,
                                floatingip_id=floatingip_id, values=values)

    # Blacklisted Zone Methods
    def create_blacklist(self, context, blacklist):
        return self.client.call(context, 'create_blacklist',
                                blacklist=blacklist)

    def get_blacklist(self, context, blacklist_id):
        return self.client.call(context, 'get_blacklist',
                                blacklist_id=blacklist_id)

    def find_blacklists(self, context, criterion=None, marker=None, limit=None,
                        sort_key=None, sort_dir=None):
        return self.client.call(
            context, 'find_blacklists', criterion=criterion, marker=marker,
            limit=limit, sort_key=sort_key, sort_dir=sort_dir)

    def update_blacklist(self, context, blacklist):
        return self.client.call(context, 'update_blacklist',
                                blacklist=blacklist)

    def delete_blacklist(self, context, blacklist_id):
        return self.client.call(context, 'delete_blacklist',
                                blacklist_id=blacklist_id)

    # Pool Server Methods
    def create_pool(self, context, pool):
        return self.client.call(context, 'create_pool', pool=pool)

    def find_pools(self, context, criterion=None, marker=None, limit=None,
                   sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_pools', criterion=criterion,
                                marker=marker, limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def find_pool(self, context, criterion=None):
        return self.client.call(context, 'find_pool', criterion=criterion)

    def get_pool(self, context, pool_id):
        return self.client.call(context, 'get_pool', pool_id=pool_id)

    def update_pool(self, context, pool):
        return self.client.call(context, 'update_pool', pool=pool)

    def delete_pool(self, context, pool_id):
        return self.client.call(context, 'delete_pool', pool_id=pool_id)

    def update_status(self, context, zone_id, status, serial, action=None):
        self.client.cast(context, 'update_status', zone_id=zone_id,
                         status=status, serial=serial, action=action)

    # Zone Ownership Transfers
    def create_zone_transfer_request(self, context, zone_transfer_request):
        return self.client.call(
            context, 'create_zone_transfer_request',
            zone_transfer_request=zone_transfer_request)

    def get_zone_transfer_request(self, context, zone_transfer_request_id):
        return self.client.call(
            context,
            'get_zone_transfer_request',
            zone_transfer_request_id=zone_transfer_request_id)

    def find_zone_transfer_requests(self, context, criterion=None, marker=None,
                                    limit=None, sort_key=None, sort_dir=None):
        return self.client.call(
            context, 'find_zone_transfer_requests', criterion=criterion,
            marker=marker, limit=limit, sort_key=sort_key, sort_dir=sort_dir)

    def update_zone_transfer_request(self, context, zone_transfer_request):
        return self.client.call(
            context, 'update_zone_transfer_request',
            zone_transfer_request=zone_transfer_request)

    def delete_zone_transfer_request(self, context, zone_transfer_request_id):
        return self.client.call(
            context,
            'delete_zone_transfer_request',
            zone_transfer_request_id=zone_transfer_request_id)

    def create_zone_transfer_accept(self, context, zone_transfer_accept):
        return self.client.call(
            context, 'create_zone_transfer_accept',
            zone_transfer_accept=zone_transfer_accept)

    def get_zone_transfer_accept(self, context, zone_transfer_accept_id):
        return self.client.call(
            context,
            'get_zone_transfer_accept',
            zone_transfer_accept_id=zone_transfer_accept_id)

    def find_zone_transfer_accepts(self, context, criterion=None, marker=None,
                                   limit=None, sort_key=None, sort_dir=None):
        return self.client.call(
            context, 'find_zone_transfer_accepts', criterion=criterion,
            marker=marker, limit=limit, sort_key=sort_key, sort_dir=sort_dir)

    def xfr_zone(self, context, zone_id):
        return self.client.call(context, 'xfr_zone', zone_id=zone_id)

    # Zone Import Methods
    def create_zone_import(self, context, request_body):
        return self.client.call(context, 'create_zone_import',
                                request_body=request_body)

    def find_zone_imports(self, context, criterion=None, marker=None,
                          limit=None, sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_zone_imports',
                                criterion=criterion, marker=marker,
                                limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def get_zone_import(self, context, zone_import_id):
        return self.client.call(context, 'get_zone_import',
                                zone_import_id=zone_import_id)

    def delete_zone_import(self, context, zone_import_id):
        return self.client.call(context, 'delete_zone_import',
                                zone_import_id=zone_import_id)

    # Zone Export Methods
    def create_zone_export(self, context, zone_id):
        return self.client.call(context, 'create_zone_export',
                                zone_id=zone_id)

    def find_zone_exports(self, context, criterion=None, marker=None,
                          limit=None, sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_zone_exports',
                                criterion=criterion, marker=marker,
                                limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def get_zone_export(self, context, zone_export_id):
        return self.client.call(context, 'get_zone_export',
                                zone_export_id=zone_export_id)

    def update_zone_export(self, context, zone_export):
        return self.client.call(context, 'update_zone_export',
                                zone_export=zone_export)

    def delete_zone_export(self, context, zone_export_id):
        return self.client.call(context, 'delete_zone_export',
                                zone_export_id=zone_export_id)

    def find_service_status(self, context, criterion=None):
        return self.client.call(context, 'find_service_status',
                                criterion=criterion)

    def find_service_statuses(self, context, criterion=None, marker=None,
                              limit=None, sort_key=None, sort_dir=None):
        return self.client.call(context, 'find_service_statuses',
                                criterion=criterion, marker=marker,
                                limit=limit, sort_key=sort_key,
                                sort_dir=sort_dir)

    def update_service_status(self, context, service_status):
        self.client.cast(context, 'update_service_status',
                         service_status=service_status)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import oslotest.base

from designate.conf import opts
from designate.conf import worker


class TestConfOpts(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_opts_tupleize(self):
        self.assertEqual([('a', 'b')], opts._tupleize({'a': 'b'}))

    def test_opts_list(self):
        self.assertIsInstance(opts.list_opts(), list)

    @mock.patch('pkgutil.iter_modules')
    def test_opts_list_module_names(self, mock_iter_modules):
        mock_iter_modules.return_value = iter(
            [
                (None, 'api', False),
                (None, 'worker', False),
                (None, 'unknown', True),
            ]
        )

        self.assertEqual(['api', 'worker'], opts._list_module_names())

    def test_opts_import_modules(self):
        self.assertEqual([worker], opts._import_modules(['worker']))

    @mock.patch('importlib.import_module')
    def test_opts_import_invalid_module(self, mock_import_module):
        mock_import_module.return_value = None

        self.assertRaisesRegex(
            Exception,
            "The module 'designate.conf.invalid' should have a 'list_opts' "
            "function which returns the config options.",
            opts._import_modules, ['invalid']
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

import oslotest.base
import testtools

from designate.api.v2 import patches
from designate import exceptions


class TestPatches(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.environ = {
            'CONTENT_TYPE': 'application/json',
        }
        self.request = patches.Request(self.environ)

    def test_unsupported_content_type(self):
        self.environ['CONTENT_TYPE'] = 'invalid'

        with testtools.ExpectedException(exceptions.UnsupportedContentType):
            self.assertIsNone(self.request.body_dict)

    @mock.patch('oslo_serialization.jsonutils.load')
    def test_request_body_empty(self, mock_load):
        mock_load.side_effect = ValueError()

        with testtools.ExpectedException(exceptions.EmptyRequestBody):
            self.assertIsNone(self.request.body_dict)

    @mock.patch('oslo_serialization.jsonutils.load')
    def test_invalid_json(self, mock_load):
        mock_load.side_effect = ValueError()
        self.request.body = b'invalid'

        with testtools.ExpectedException(exceptions.InvalidJson):
            self.assertIsNone(self.request.body_dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os

from designate.manage import tlds
from designate.tests import base_fixtures
import designate.tests.functional
from designate.tests import resources


def get_tlds_path(name='tlds_list'):
    return os.path.join(resources.path, 'tlds', name)


class ManageTLDSTestCase(designate.tests.functional.TestCase):
    def setUp(self):
        super().setUp()
        self.stdlog = base_fixtures.StandardLogging()
        self.useFixture(self.stdlog)
        self.tlds = tlds.TLDCommands()

    def test_import_tlds(self):
        self.tlds.from_file(get_tlds_path(), ',')

        tlds = self.central_service.find_tlds(self.admin_context)
        self.assertEqual(5, len(tlds))
        for tld in tlds:
            self.assertFalse(tld.description)

        self.assertNotIn('Number of errors', self.stdlog.logger.output)

    def test_import_tlds_with_descriptions(self):
        self.tlds.from_file(get_tlds_path('tlds_list_with_descriptions'), ',')

        tlds = self.central_service.find_tlds(self.admin_context)
        self.assertEqual(5, len(tlds))
        for tld in tlds:
            self.assertTrue(tld.description)

        self.assertNotIn('Number of errors', self.stdlog.logger.output)

    def test_import_tlds_with_extra_fields(self):
        self.tlds.from_file(get_tlds_path('tlds_list_with_extra_fields'), ',')

        tlds = self.central_service.find_tlds(self.admin_context)
        self.assertEqual(0, len(tlds))

        self.assertIn('InvalidLine -->', self.stdlog.logger.output)
        self.assertIn('Number of errors', self.stdlog.logger.output)

    def test_import_tlds_with_invalid_tlds(self):
        self.tlds.from_file(get_tlds_path('tlds_list_with_invalid_tlds'), ',')

        tlds = self.central_service.find_tlds(self.admin_context)
        self.assertEqual(1, len(tlds))

        self.assertIn('InvalidTld -->', self.stdlog.logger.output)
        self.assertIn('InvalidDescription -->', self.stdlog.logger.output)
        self.assertIn('DuplicateTld -->', self.stdlog.logger.output)
        self.assertIn('Number of errors', self.stdlog.logger.output)

    def test_import_tlds_file_does_not_exist(self):
        self.assertRaisesRegex(
            Exception,
            'TLD Input file Not Found',
            self.tlds.from_file, get_tlds_path('invalid_file'), ','
        )



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Endre Karlson <endre.karlson@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_config import cfg
from oslo_log import log

from designate.network_api import base


LOG = log.getLogger(__name__)

neutron_opts = [
    cfg.StrOpt('network_api', default='neutron', help='Which API to use.')
]

cfg.CONF.register_opts(neutron_opts)


def get_network_api(network_api_driver):
    LOG.debug('Loading network_api driver: %s', network_api_driver)

    cls = base.NetworkAPI.get_driver(network_api_driver)

    return cls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import oslotest.base

from designate.api.v2.controllers import floatingips
from designate import exceptions


class FloatingIPTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()

    def test_fip_key_to_data(self):
        self.assertEqual(
            ('RegionOne', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'RegionOne:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )
        self.assertEqual(
            ('RegionTwo', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'RegionTwo:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )

        self.assertEqual(
            ('region-one', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'region-one:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )
        self.assertEqual(
            ('region-1', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'region-1:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )
        self.assertEqual(
            ('region-1-test', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'region-1-test:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )
        self.assertEqual(
            ('region.1.test', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'region.1.test:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )
        self.assertEqual(
            ('region.test', '2fc6745d-1631-4f34-b13d-90f9014236c0'),
            floatingips.fip_key_to_data(
                'region.test:2fc6745d-1631-4f34-b13d-90f9014236c0'
            )
        )

    def test_fip_key_to_data_bad_request(self):
        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Floating IP invalid is not in the format of <region>:<uuid>',
            floatingips.fip_key_to_data, 'invalid'
        )
        self.assertRaisesRegex(
            exceptions.BadRequest,
            'Floating IP :2fc6745d-1631-4f34-b13d-90f9014236c0 is not in the '
            'format of <region>:<uuid>',
            floatingips.fip_key_to_data,
            ':2fc6745d-1631-4f34-b13d-90f9014236c0'
        )



# Copyright 2016 Rackspace Inc.
#
# Author: Eric Larson <eric.larson@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

import futurist
from oslo_log import log as logging

import designate.conf
from designate import exceptions


LOG = logging.getLogger(__name__)
CONF = designate.conf.CONF


def default_executor():
    # TODO(mugsie): if (when) we move away from eventlet this may have to
    # revert back to ThreadPoolExecutor - this is changing due to
    # https://bugs.launchpad.net/bugs/1782647 (eventlet + py37 issues)
    return futurist.GreenThreadPoolExecutor(CONF['service:worker'].threads)


class Executor:
    """
    Object to facilitate the running of a task, or a set of tasks on an
    executor that can map multiple tasks across a configurable number of
    threads
    """

    def __init__(self, executor=None):
        self._executor = executor or default_executor()

    @staticmethod
    def do(task):
        try:
            return task()
        except exceptions.BadAction as e:
            LOG.warning(e)

    @staticmethod
    def task_name(task):
        if hasattr(task, 'task_name'):
            return str(task.task_name)
        if hasattr(task, 'func_name'):
            return str(task.func_name)
        return 'UnnamedTask'

    def run(self, tasks):
        """
        Run task or set of tasks
        :param tasks: the task or tasks you want to execute in the
                      executor's pool

        :return: The results of the tasks (list)

        If a single task is pass
        """
        if callable(tasks):
            tasks = [tasks]

        start_time = time.monotonic()
        results = [r for r in self._executor.map(self.do, tasks)]
        elapsed_time = time.monotonic() - start_time

        LOG.debug(
            'Finished Task(s): %(tasks)s in %(time)fs',
            {
                'tasks': ', '.join([self.task_name(t) for t in tasks]),
                'time': elapsed_time
            }
        )

        return results



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 Justin Santa Barbara
# Copyright 2011 OpenStack Foundation
# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import errno
import socket
import struct
import threading

import eventlet.debug
from oslo_log import log as logging
import oslo_messaging as messaging
from oslo_service import service
from oslo_service import sslutils
from oslo_service import wsgi
from oslo_utils import netutils

from designate.common.decorators import rpc as rpc_decorator
from designate.common import profiler
import designate.conf
from designate.i18n import _
from designate import policy
from designate import rpc
from designate import utils
from designate import version

CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class Service(service.Service):
    def __init__(self, name, threads=None):
        threads = threads or 1000
        super().__init__(threads)
        self.name = name
        self.host = CONF.host

        policy.init()

        if not rpc.initialized():
            rpc.init(CONF)

        profiler.setup_profiler((''.join(('designate-', self.name))),
                                self.host)

    def start(self):
        LOG.info('Starting %(name)s service (version: %(version)s)',
                 {
                     'name': self.name,
                     'version': version.version_info.version_string()
                 })
        super().start()

    def stop(self, graceful=True):
        LOG.info('Stopping %(name)s service', {'name': self.name})
        super().stop(graceful)


class RPCService(Service):
    def __init__(self, name, rpc_topic, threads=None):
        super().__init__(name, threads)
        LOG.debug("Creating RPC Server on topic '%s' for %s",
                  rpc_topic, self.name)

        self.endpoints = [self]
        self.exception_thread_local = rpc_decorator.ExceptionThreadLocal()
        self.notifier = None
        self.rpc_server = None
        self.rpc_topic = rpc_topic

    def start(self):
        super().start()
        target = messaging.Target(topic=self.rpc_topic, server=self.host)
        self.rpc_server = rpc.get_server(target, self.endpoints)
        self.rpc_server.start()
        self.notifier = rpc.get_notifier(self.name)

    def stop(self, graceful=True):
        if self.rpc_server:
            self.rpc_server.stop()
        super().stop(graceful)

    def wait(self):
        super().wait()


class WSGIService(Service):
    def __init__(self, app, name, listen, max_url_len=None):
        super().__init__(name)
        self.app = app
        self.name = name

        self.listen = listen

        self.servers = []

        for address in self.listen:
            host, port = netutils.parse_host_port(address)
            server = wsgi.Server(
                CONF, name, app,
                host=host,
                port=port,
                pool_size=CONF['service:api'].threads,
                backlog=CONF.backlog,
                use_ssl=sslutils.is_enabled(CONF),
                max_url_len=max_url_len
            )

            self.servers.append(server)

    def start(self):
        for server in self.servers:
            server.start()
        super().start()

    def stop(self, graceful=True):
        for server in self.servers:
            server.stop()
        super().stop(graceful)

    def wait(self):
        for server in self.servers:
            server.wait()
        super().wait()


class DNSService:
    _TCP_RECV_MAX_SIZE = 65535

    def __init__(self, app, tg, listen, tcp_backlog, tcp_keepidle,
                 tcp_recv_timeout):
        self._running = threading.Event()
        self.app = app
        self.tg = tg
        self.tcp_backlog = tcp_backlog
        self.tcp_keepidle = tcp_keepidle
        self.tcp_recv_timeout = tcp_recv_timeout
        self.listen = listen

        # Eventet will complain loudly about our use of multiple greentheads
        # reading/writing to the UDP socket at once. Disable this warning.
        eventlet.debug.hub_prevent_multiple_readers(False)

        self._dns_socks_tcp = []
        self._dns_socks_udp = []

    def start(self):
        self._running.set()

        addresses = map(
            netutils.parse_host_port,
            set(self.listen)
        )

        for address in addresses:
            self._start(address[0], address[1])

    def _start(self, host, port):
        sock_tcp = utils.bind_tcp(
            host, port, self.tcp_backlog, self.tcp_keepidle
        )
        sock_udp = utils.bind_udp(
            host, port
        )

        self._dns_socks_tcp.append(sock_tcp)
        self._dns_socks_udp.append(sock_udp)

        self.tg.add_thread(self._dns_handle_tcp, sock_tcp)
        self.tg.add_thread(self._dns_handle_udp, sock_udp)

    def stop(self):
        self._running.clear()

        for sock_tcp in self._dns_socks_tcp:
            sock_tcp.close()

        for sock_udp in self._dns_socks_udp:
            sock_udp.close()

    def _dns_handle_tcp(self, sock_tcp):
        LOG.info('_handle_tcp thread started')

        client = None
        while self._running.is_set():
            addr = None
            try:
                # handle a new TCP connection
                client, addr = sock_tcp.accept()

                if self.tcp_recv_timeout:
                    client.settimeout(self.tcp_recv_timeout)

                LOG.debug(
                    'Handling TCP Request from: %(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1]
                    }
                )
                if len(addr) == 4:
                    LOG.debug(
                        'Flow info: %(host)s scope: %(port)d',
                        {
                            'host': addr[2],
                            'port': addr[3]
                        }
                    )

                # Dispatch a thread to handle the connection
                self.tg.add_thread(self._dns_handle_tcp_conn, addr, client)

            # NOTE: Any uncaught exceptions will result in the main loop
            # ending unexpectedly. Ensure proper ordering of blocks, and
            # ensure no exceptions are generated from within.
            except socket.timeout:
                pass
            except OSError as e:
                if client:
                    client.close()
                errname = errno.errorcode[e.args[0]]
                addr = addr or (None, 0)
                LOG.warning(
                    'Socket error %(err)s from: %(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1],
                        'err': errname
                    }
                )
            except Exception:
                if client:
                    client.close()
                addr = addr or (None, 0)
                LOG.exception(
                    'Unknown exception handling TCP request from: '
                    '%(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1]
                    }
                )

    def _dns_handle_tcp_conn(self, addr, client):
        """
        Handle a DNS Query over TCP. Multiple queries can be pipelined
        through the same TCP connection but they will be processed
        sequentially.
        See https://tools.ietf.org/html/draft-ietf-dnsop-5966bis-03
        Raises no exception: it's to be run in an eventlet green thread

        :param addr: Tuple of the client's (IPv4 addr, Port) or
                     (IPv6 addr, Port, Flow info, Scope ID)
        :type addr: tuple
        :param client: Client socket
        :type client: socket.socket
        :raises: None
        """
        host, port = addr[:2]
        try:
            # The whole loop lives in a try/except block. On exceptions, the
            # connection is closed: there would be little chance to save
            # the connection after a struct error, a socket error.
            while True:
                # Decode the first 2 bytes containing the query length
                expected_length_raw = client.recv(2)
                if len(expected_length_raw) == 0:
                    break
                (expected_length,) = struct.unpack('!H', expected_length_raw)

                # Keep receiving data until we've got all the data we expect
                # The buffer contains only one query at a time
                buf = b''
                while len(buf) < expected_length:
                    recv_size = min(expected_length - len(buf),
                                    self._TCP_RECV_MAX_SIZE)
                    data = client.recv(recv_size)
                    if not data:
                        break
                    buf += data

                query = buf

                # Call into the DNS Application itself with payload and addr
                for response in self.app({'payload': query, 'addr': addr}):

                    # Send back a response only if present
                    if response is None:
                        continue

                    # Handle TCP Responses
                    msg_length = len(response)
                    tcp_response = struct.pack("!H", msg_length) + response
                    client.sendall(tcp_response)

        except socket.timeout:
            LOG.info(
                'TCP Timeout from: %(host)s:%(port)d',
                {
                    'host': host,
                    'port': port
                }
            )
        except OSError as e:
            errname = errno.errorcode[e.args[0]]
            LOG.warning(
                'Socket error %(err)s from: %(host)s:%(port)d',
                {
                    'host': host,
                    'port': port,
                    'err': errname
                }
            )
        except struct.error:
            LOG.warning(
                'Invalid packet from: %(host)s:%(port)d',
                {
                    'host': host,
                    'port': port
                }
            )
        except Exception:
            LOG.exception(
                'Unknown exception handling TCP request from: '
                '%(host)s:%(port)d',
                {
                    'host': host,
                    'port': port
                }
            )
        finally:
            client.close()

    def _dns_handle_udp(self, sock_udp):
        """Handle a DNS Query over UDP in a dedicated thread

        :param sock_udp: UDP socket
        :type sock_udp: socket.socket
        :raises: None
        """
        LOG.info('_handle_udp thread started')

        while self._running.is_set():
            addr = None
            try:
                # TODO(kiall): Determine the appropriate default value for
                #              UDP recvfrom.
                payload, addr = sock_udp.recvfrom(8192)

                LOG.debug(
                    'Handling UDP Request from: %(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1]
                    }
                )

                # Dispatch a thread to handle the query
                self.tg.add_thread(self._dns_handle_udp_query, sock_udp, addr,
                                   payload)
            except socket.timeout:
                pass
            except OSError as e:
                errname = errno.errorcode[e.args[0]]
                addr = addr or (None, 0)
                LOG.warning(
                    'Socket error %(err)s from: %(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1],
                        'err': errname
                    }
                )
            except Exception:
                addr = addr or (None, 0)
                LOG.exception(
                    'Unknown exception handling UDP request from: '
                    '%(host)s:%(port)d',
                    {
                        'host': addr[0],
                        'port': addr[1]
                    }
                )

    def _dns_handle_udp_query(self, sock, addr, payload):
        """
        Handle a DNS Query over UDP

        :param sock: UDP socket
        :type sock: socket.socket
        :param addr: Tuple of the client's (IP, Port)
        :type addr: tuple
        :param payload: Raw DNS query payload
        :type payload: string
        :raises: None
        """
        try:
            # Call into the DNS Application itself with the payload and addr
            for response in self.app({'payload': payload, 'addr': addr}):
                if response is not None:
                    sock.sendto(response, addr)
        except Exception:
            LOG.exception(
                'Unhandled exception while processing request from '
                '%(host)s:%(port)d',
                {
                    'host': addr[0],
                    'port': addr[1]
                }
            )


_launcher = None


def serve(server, workers=None):
    global _launcher
    if _launcher:
        raise RuntimeError(_('serve() can only be called once'))

    _launcher = service.launch(CONF, server, workers=workers,
                               restart_method='mutate')


def wait():
    try:
        _launcher.wait()
    except KeyboardInterrupt:
        LOG.debug('Caught KeyboardInterrupt, shutting down now')
    rpc.cleanup()



# Copyright 2015 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspae.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import pecan

from designate.api.v2.controllers import rest
from designate.common import constants
from designate import exceptions
from designate.objects.adapters import DesignateAdapter
from designate import policy
from designate import utils

LOG = logging.getLogger(__name__)


class ZoneExportController(rest.RestController):
    @pecan.expose(template=None, content_type='text/dns')
    @utils.validate_uuid('export_id')
    def get_all(self, export_id):
        context = pecan.request.environ['context']
        target = {constants.RBAC_PROJECT_ID: context.project_id,
                  'tenant_id': context.project_id}

        policy.check('zone_export', context, target)

        export = self.central_api.get_zone_export(context, export_id)

        if export.location and export.location.startswith('designate://'):
            return self.central_api.export_zone(context, export['zone_id'])
        else:
            raise exceptions.BadRequest(
                'Zone can not be exported synchronously'
            )


class ZoneExportCreateController(rest.RestController):

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id')
    def post_all(self, zone_id):
        """Create Zone Export"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        # Create the zone_export
        zone_export = self.central_api.create_zone_export(
            context, zone_id)
        response.status_int = 202

        LOG.info("Created %(zone_export)s", {'zone_export': zone_export})

        zone_export = DesignateAdapter.render('API_v2', zone_export,
                                              request=request)

        response.headers['Location'] = zone_export['links']['self']
        return zone_export


class ZoneExportsController(rest.RestController):

    SORT_KEYS = ['created_at', 'id', 'updated_at']

    export = ZoneExportController()

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('export_id')
    def get_one(self, export_id):
        """Get Zone Exports"""

        request = pecan.request
        context = request.environ['context']

        zone_export = self.central_api.get_zone_export(context, export_id)

        LOG.info("Retrieved %(export)s", {'export': zone_export})

        return DesignateAdapter.render('API_v2', zone_export, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    def get_all(self, **params):
        """List Zone Exports"""
        request = pecan.request
        context = request.environ['context']
        marker, limit, sort_key, sort_dir = utils.get_paging_params(
                context, params, self.SORT_KEYS)

        # Extract any filter params.
        accepted_filters = ('status', 'message', 'zone_id', )

        criterion = self._apply_filter_params(
            params, accepted_filters, {})

        zone_exports = self.central_api.find_zone_exports(
            context, criterion, marker, limit, sort_key, sort_dir)

        LOG.info("Retrieved %(exports)s",
                 {'exports': zone_exports})

        return DesignateAdapter.render('API_v2', zone_exports, request=request)

    # NOTE: template=None is important here, template='json:' manifests
    #       in this bug: https://bugs.launchpad.net/designate/+bug/1592153
    @pecan.expose(template=None, content_type='application/json')
    @utils.validate_uuid('zone_export_id')
    def delete_one(self, zone_export_id):
        """Delete Zone Export"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        zone_export = self.central_api.delete_zone_export(
            context, zone_export_id)

        LOG.info("Deleted %(zone_export)s", {'zone_export': zone_export})

        response.status_int = 204

        return ''



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from unittest import mock

from oslo_config import fixture as cfg_fixture
import oslotest.base

from designate.api.admin.controllers import root as admin_root
from designate.api.v2.controllers import root as v2_root
import designate.conf
import designate.tests


CONF = designate.conf.CONF


class RootTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_admin_root(self, mock_manger):
        mock_extension = mock.Mock()
        mock_extension.obj.get_path.return_value = 'a'
        mock_manger.return_value = [mock_extension]
        CONF.set_override(
            'enabled_extensions_admin',
            ['admin'],
            'service:api'
        )
        admin_root.RootController()

        mock_manger.assert_called_with(
            namespace=mock.ANY, names=['admin'], invoke_on_load=True
        )

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_admin_root_no_extensions(self, mock_manger):
        mock_manger.return_value = []
        CONF.set_override(
            'enabled_extensions_admin',
            [],
            'service:api'
        )
        admin_root.RootController()

        mock_manger.assert_not_called()

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_v2_root(self, mock_manger):
        mock_extension = mock.Mock()
        mock_extension.obj.get_path.return_value = 'a'
        mock_manger.return_value = [mock_extension]
        CONF.set_override(
            'enabled_extensions_v2',
            ['v2'],
            'service:api'
        )
        v2_root.RootController()

        mock_manger.assert_called_with(
            namespace=mock.ANY, names=['v2'], invoke_on_load=True
        )

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_v2_root_object_not_found(self, mock_manger):
        mock_extension = mock.Mock()
        mock_extension.obj.get_path.return_value = '..test.a'
        mock_manger.return_value = [mock_extension]
        CONF.set_override(
            'enabled_extensions_v2',
            ['v2'],
            'service:api'
        )

        self.assertRaisesRegex(
            AttributeError,
            "object has no attribute 'test'",
            v2_root.RootController
        )

        mock_manger.assert_called_with(
            namespace=mock.ANY, names=['v2'], invoke_on_load=True
        )

    @mock.patch('stevedore.named.NamedExtensionManager')
    def test_v2_root_no_extensions(self, mock_manger):
        mock_manger.return_value = []
        CONF.set_override(
            'enabled_extensions_v2',
            [],
            'service:api'
        )
        v2_root.RootController()

        mock_manger.assert_not_called()



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Federico Ceratto <federico.ceratto@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import itertools
from unittest import mock

from oslo_config import fixture as cfg_fixture
from oslo_log import log as logging
import oslotest.base


import designate.conf
from designate import exceptions
from designate import objects
from designate.objects.adapters import DesignateAdapter


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


def create_test_recordset():
    record_set = objects.RecordSet(
        id='f6a2cbd6-7f9a-4e0c-a00d-98a02aa73fc8',
        zone_id='74038683-cab1-4056-bdf8-b39bd155ff21',
        name='www.example.org.',
        type='A',
        records=objects.RecordList(objects=[
            objects.Record(data='192.0.2.1'),
            objects.Record(data='192.0.2.2'),
        ]),
        shard=0
    )
    return record_set


class RecordSetTest(oslotest.base.BaseTestCase):
    def setUp(self):
        super().setUp()
        self.useFixture(cfg_fixture.Config(CONF))

    def test_init(self):
        record_set = create_test_recordset()
        self.assertEqual('www.example.org.', record_set.name)
        self.assertEqual('A', record_set.type)

    def test_to_repr(self):
        record_set = create_test_recordset()
        self.assertEqual(
            "<RecordSet id:'f6a2cbd6-7f9a-4e0c-a00d-98a02aa73fc8' type:'A' "
            "name:'www.example.org.' "
            "zone_id:'74038683-cab1-4056-bdf8-b39bd155ff21' shard:'0'>",
            repr(record_set)
        )

    def test_not_managed(self):
        record_set = create_test_recordset()
        self.assertFalse(record_set.managed)

    def test_managed(self):
        record_set = objects.RecordSet(
            name='www.example.org.',
            type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', managed=True),
                objects.Record(data='192.0.2.2'),
            ])
        )
        self.assertTrue(record_set.managed)

    def test_action(self):
        action = 'CREATE'
        record_set = objects.RecordSet(
            name='www.example.org.',
            type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', action=action),
            ])
        )
        self.assertEqual(action, record_set.action)

    def test_action_create(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', action='CREATE'),
            ])
        )
        self.assertEqual('CREATE', record_set.action)

    def test_action_create_plus_update(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', action='CREATE'),
                objects.Record(data='192.0.2.2', action='UPDATE'),
            ])
        )
        self.assertEqual('UPDATE', record_set.action)

    def test_action_delete_plus_update(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', action='DELETE'),
                objects.Record(data='192.0.2.2', action='UPDATE'),
            ])
        )
        self.assertEqual('UPDATE', record_set.action)

    def test_action_delete_only(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.1', action='DELETE'),
                objects.Record(data='192.0.2.2', action='DELETE'),
            ])
        )
        self.assertEqual('DELETE', record_set.action)

    def test_status_error(self):
        statuses = ('ERROR', 'PENDING', 'ACTIVE', 'DELETED')
        for s1, s2, s3, s4 in itertools.permutations(statuses):
            record_set = objects.RecordSet(
                name='www.example.org.', type='A',
                records=objects.RecordList(objects=[
                    objects.Record(data='192.0.2.1', status=s1),
                    objects.Record(data='192.0.2.2', status=s2),
                    objects.Record(data='192.0.2.3', status=s3),
                    objects.Record(data='192.0.2.4', status=s4),
                ])
            )
            self.assertEqual(record_set.status, 'ERROR')

    def test_status_pending(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.2', status='PENDING'),
                objects.Record(data='192.0.2.3', status='ACTIVE'),
            ])
        )
        self.assertEqual('PENDING', record_set.status)

    def test_status_pending2(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.3', status='ACTIVE'),
                objects.Record(data='192.0.2.2', status='PENDING'),
            ])
        )
        self.assertEqual('PENDING', record_set.status)

    def test_status_active(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.3', status='ACTIVE'),
            ])
        )
        self.assertEqual('ACTIVE', record_set.status)

    def test_status_deleted(self):
        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.2', status='DELETED'),
            ])
        )
        self.assertEqual('DELETED', record_set.status)

    def test_status_many_expect_error(self):
        rs = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.2', status='ACTIVE'),
                objects.Record(data='192.0.2.3', status='DELETED'),
                objects.Record(data='192.0.2.4', status='DELETED'),
                objects.Record(data='192.0.2.5', status='DELETED'),
                objects.Record(data='192.0.2.6', status='ACTIVE'),
                objects.Record(data='192.0.2.7', status='ACTIVE'),
                objects.Record(data='192.0.2.8', status='ERROR'),
                objects.Record(data='192.0.2.9', status='ACTIVE'),
                objects.Record(data='192.0.2.10', status='ACTIVE'),
            ])
        )
        self.assertEqual('ERROR', rs.status)

    def test_status_many_expect_pending(self):
        rs = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.2', status='ACTIVE'),
                objects.Record(data='192.0.2.3', status='DELETED'),
                objects.Record(data='192.0.2.4', status='PENDING'),
                objects.Record(data='192.0.2.5', status='DELETED'),
                objects.Record(data='192.0.2.6', status='PENDING'),
                objects.Record(data='192.0.2.7', status='ACTIVE'),
                objects.Record(data='192.0.2.8', status='DELETED'),
                objects.Record(data='192.0.2.9', status='PENDING'),
                objects.Record(data='192.0.2.10', status='ACTIVE'),
            ])
        )
        self.assertEqual('PENDING', rs.status)

    def test_status_many_expect_active(self):
        rs = objects.RecordSet(
            name='www.example.org.', type='A',
            records=objects.RecordList(objects=[
                objects.Record(data='192.0.2.2', status='ACTIVE'),
                objects.Record(data='192.0.2.3', status='DELETED'),
                objects.Record(data='192.0.2.4', status='DELETED'),
                objects.Record(data='192.0.2.5', status='DELETED'),
                objects.Record(data='192.0.2.6', status='ACTIVE'),
                objects.Record(data='192.0.2.7', status='ACTIVE'),
                objects.Record(data='192.0.2.8', status='DELETED'),
                objects.Record(data='192.0.2.9', status='ACTIVE'),
                objects.Record(data='192.0.2.10', status='ACTIVE'),
            ])
        )
        self.assertEqual('ACTIVE', rs.status)

    def test_validate(self):
        record_set = create_test_recordset()
        record_set.validate()

    def test_validate_attribute_error(self):
        record_set = objects.RecordSet(
            name='www.example.org.',
            type='NAPTR',
            records=objects.RecordList(objects=[
                objects.Record(data=None),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            record_set.validate
        )

    def test_validate_type_error(self):
        record_set = objects.RecordSet(
            name='www.example.org.',
            type='TXT',
            records=objects.RecordList(objects=[
                objects.Record(data=None),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            record_set.validate
        )

    def test_validate_unsupported_recordset_type(self):
        CONF.set_override('supported_record_type', ['A', 'AAAA', 'CNAME'])

        record_set = objects.RecordSet(
            name='www.example.org.', type='PTR',
            records=[]
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            record_set.validate
        )

    def test_validate_handle_exception(self):
        record_set = create_test_recordset()
        rs_module = record_set.__class__.__bases__[0].__module__
        fn_name = f'{rs_module}.DesignateObject.obj_cls_from_name'

        with mock.patch(fn_name) as patched:
            patched.side_effect = KeyError
            self.assertRaisesRegex(
                exceptions.InvalidObject,
                'Provided object does not match schema',
                record_set.validate
            )

    def test_parse_rrset_object_preserves_changes(self):
        old_ip = '1.1.1.1'
        new_ip = '8.8.8.8'
        original_records = objects.RecordList(
            objects=[
                objects.Record(data=old_ip),
            ]
        )

        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=original_records
        )

        body = {
            'records': [
                new_ip
            ]
        }

        record_set = DesignateAdapter.parse('API_v2', body, record_set)
        self.assertIn('records', record_set.obj_what_changed())

        def get_data(record_list):
            return {r.data for r in record_list}

        self.assertEqual(
            {old_ip},
            get_data(record_set.obj_get_original_value('records'))
        )

        self.assertEqual(
            {new_ip},
            get_data(record_set.obj_get_changes()['records'])
        )

    def test_parse_rrset_object_preserves_changes_multiple_rrs(self):
        old_ips = ['1.1.1.1', '2.2.2.2']
        new_ips = ['2.2.2.2', '8.8.8.8']
        original_records = objects.RecordList(
            objects=[
                objects.Record(data=ip) for ip in old_ips
            ]
        )

        record_set = objects.RecordSet(
            name='www.example.org.', type='A',
            records=original_records
        )

        body = {
            'records': new_ips
        }

        record_set = DesignateAdapter.parse('API_v2', body, record_set)
        self.assertIn('records', record_set.obj_what_changed())

        def get_data(record_list):
            return {r.data for r in record_list}

        self.assertEqual(
            set(old_ips),
            get_data(record_set.obj_get_original_value('records'))
        )

        self.assertEqual(
            set(new_ips),
            get_data(record_set.obj_get_changes()['records'])
        )



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_log import versionutils
from oslo_policy import policy

from designate.common import constants
from designate.common.policies import base

DEPRECATED_REASON = """
The zone export API now supports system scope and default roles.
"""

deprecated_zone_export = policy.DeprecatedRule(
    name="zone_export",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_create_zone_export = policy.DeprecatedRule(
    name="create_zone_export",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_find_zone_exports = policy.DeprecatedRule(
    name="find_zone_exports",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_get_zone_export = policy.DeprecatedRule(
    name="get_zone_export",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_update_zone_export = policy.DeprecatedRule(
    name="update_zone_export",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_delete_zone_export = policy.DeprecatedRule(
    name="delete_zone_export",
    check_str=base.RULE_ADMIN_OR_OWNER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)


rules = [
    policy.DocumentedRuleDefault(
        name="zone_export",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Retrive a Zone Export from the Designate Datastore",
        operations=[
            {
                'path': '/v2/zones/tasks/exports/{zone_export_id}/export',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_zone_export
    ),
    policy.DocumentedRuleDefault(
        name="create_zone_export",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Create Zone Export",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/tasks/export',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_create_zone_export
    ),
    policy.DocumentedRuleDefault(
        name="find_zone_exports",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        description="List Zone Exports",
        operations=[
            {
                'path': '/v2/zones/tasks/exports',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_find_zone_exports
    ),
    policy.DocumentedRuleDefault(
        name="get_zone_export",
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=[constants.PROJECT],
        description="Get Zone Exports",
        operations=[
            {
                'path': '/v2/zones/tasks/exports/{zone_export_id}',
                'method': 'GET'
            }
        ],
        deprecated_rule=deprecated_get_zone_export
    ),
    policy.DocumentedRuleDefault(
        name="update_zone_export",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Update Zone Exports",
        operations=[
            {
                'path': '/v2/zones/{zone_id}/tasks/export',
                'method': 'POST'
            }
        ],
        deprecated_rule=deprecated_update_zone_export
    ),
    policy.DocumentedRuleDefault(
        name="delete_zone_export",
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=[constants.PROJECT],
        description="Delete a zone export",
        operations=[
            {
                'path': '/v2/zones/tasks/exports/{zone_export_id}',
                'method': 'DELETE'
            }
        ],
        deprecated_rule=deprecated_delete_zone_export
    ),
]


def list_rules():
    return rules



# Copyright (c) 2014 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging
from stevedore import named

from designate.api.v2.controllers import errors
import designate.conf


CONF = designate.conf.CONF
LOG = logging.getLogger(__name__)


class RootController:
    """
    This is /admin/ Controller. Pecan will find all controllers via the object
    properties attached to this.
    """

    def __init__(self):
        enabled_ext = CONF['service:api'].enabled_extensions_admin
        if len(enabled_ext) > 0:
            self._mgr = named.NamedExtensionManager(
                namespace='designate.api.admin.extensions',
                names=enabled_ext,
                invoke_on_load=True)
            for ext in self._mgr:
                controller = self
                path = ext.obj.get_path()
                LOG.info("Registering an API extension at path %s", path)
                for p in path.split('.')[:-1]:
                    if p != '':
                        controller = getattr(controller, p)
                setattr(controller, path.split('.')[-1], ext.obj)

    errors = errors.ErrorsController()



# Copyright 2018 Canonical Ltd.
#
# Author: Tytus Kurek <tytus.kurek@canonical.com>
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging
import oslotest.base

from designate import objects

LOG = logging.getLogger(__name__)


class RRDataNAPTRTest(oslotest.base.BaseTestCase):
    def test_parse_naptr(self):
        naptr_record = objects.NAPTR()
        naptr_record.from_string(
            '0 0 S SIP+D2U !^.*$!sip:customer-service@example.com! _sip._udp.example.com.')  # noqa

        self.assertEqual(0, naptr_record.order)
        self.assertEqual(0, naptr_record.preference)
        self.assertEqual('S', naptr_record.flags)
        self.assertEqual('SIP+D2U', naptr_record.service)
        self.assertEqual('!^.*$!sip:customer-service@example.com!',
                         naptr_record.regexp)
        self.assertEqual('_sip._udp.example.com.', naptr_record.replacement)

    def test_parse_naptr_quoted(self):
        naptr_record = objects.NAPTR()
        naptr_record.from_string(
            '0 0 "S" "SIP+D2U" "!^.*$!sip:support@example.com!" _sip._udp.example.com.')  # noqa

        self.assertEqual(0, naptr_record.order)
        self.assertEqual(0, naptr_record.preference)
        self.assertEqual('S', naptr_record.flags)
        self.assertEqual('SIP+D2U', naptr_record.service)
        self.assertEqual('!^.*$!sip:support@example.com!',
                         naptr_record.regexp)
        self.assertEqual('_sip._udp.example.com.', naptr_record.replacement)

    def test_parse_naptr_empty_fields(self):
        naptr_record = objects.NAPTR()
        naptr_record.from_string('0 0 "" "" "" test.')  # noqa

        self.assertEqual(0, naptr_record.order)
        self.assertEqual(0, naptr_record.preference)
        self.assertEqual('', naptr_record.flags)
        self.assertEqual('', naptr_record.service)
        self.assertEqual('',
                         naptr_record.regexp)
        self.assertEqual('test.', naptr_record.replacement)

    def test_parse_naptr_valid_exampe1(self):
        naptr_record = objects.NAPTR()
        naptr_record.from_string('65535 65535 "SAUP" "bloop" ":beep::" test.')  # noqa

        self.assertEqual(65535, naptr_record.order)
        self.assertEqual(65535, naptr_record.preference)
        self.assertEqual('SAUP', naptr_record.flags)
        self.assertEqual('bloop', naptr_record.service)
        self.assertEqual(':beep::',
                         naptr_record.regexp)
        self.assertEqual('test.', naptr_record.replacement)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import inspect
import threading

from oslo_messaging.rpc import dispatcher as rpc_dispatcher

import designate.exceptions


RPC_LOGGING_DISALLOW = [
    'get_instance',
    '__init__'
]


class ExceptionThreadLocal(threading.local):
    def __init__(self):
        super().__init__()
        self.depth = 0

    def reset_depth(self):
        self.depth = 0


def expected_exceptions():
    def outer(f):
        @functools.wraps(f)
        def exception_wrapper(cls, *args, **kwargs):
            cls.exception_thread_local.depth += 1

            # We only want to wrap the first function wrapped.
            if cls.exception_thread_local.depth > 1:
                return f(cls, *args, **kwargs)

            try:
                return f(cls, *args, **kwargs)
            except designate.exceptions.DesignateException as e:
                if e.expected:
                    raise rpc_dispatcher.ExpectedException()
                raise
            finally:
                cls.exception_thread_local.reset_depth()
        return exception_wrapper
    return outer


def log_rpc_call(func, rpcapi, logger):
    def wrapped(*args, **kwargs):
        logger.debug(
            'Calling designate.%(rpcapi)s.%(function)s() over RPC',
            {
                'function': func.__name__,
                'rpcapi': rpcapi
            }
        )
        return func(*args, **kwargs)
    return wrapped


def rpc_logging(logger, rpcapi):
    def wrapper(cls):
        disallow = getattr(cls, 'RPC_LOGGING_DISALLOW', [])
        disallow.extend(RPC_LOGGING_DISALLOW)
        for name, value in inspect.getmembers(cls, inspect.ismethod):
            if name not in disallow:
                setattr(cls, name, log_rpc_call(value, rpcapi, logger))
        return cls
    return wrapper
